-- DailyQuestHelper Version 0.1
-- author alexwild
DailyQuestHelper = LibStub("AceAddon-3.0"):NewAddon("DailyQuestHelper", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("DailyQuestHelper")

local general_options;
local quest_options;
local profileDB;

local function Debug(...)
    local s = "Profile debugmessage:";
    	for i=1,select("#", ...) do
		local x = select(i, ...)
		s = strjoin(" ",s,tostring(x))
	end
	DEFAULT_CHAT_FRAME:AddMessage(s)
end

----------------------------------------------------------
-- addon initialization
----------------------------------------------------------
function DailyQuestHelper:OnInitialize()

	self.databaseVersion = GetAddOnMetadata("DailyQuestHelper","X-Database-Version") or "";

	local defaults = {
        profile = {
            debug = false,
            enable = true,
			shownewquests = true,
			quests = {},			
        },			
    }
    
	self.db = LibStub("AceDB-3.0"):New("DailyQuestHelperDB", defaults, "Default");	
	self.db.RegisterCallback(self, "OnNewProfile", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
		
	if (self.db.char.version == nil or self.db.char.version < self.databaseVersion) then
		DailyQuestHelper:ConvertOldDB();
	end
	
	profileDB = self.db.profile;
	
    self.version = GetAddOnMetadata("DailyQuestHelper","X-Curse-Packaged-Version") or "";
	
	local ACFG = LibStub("AceConfig-3.0");
	quest_options = DailyQuestHelper:CreateDynamicQuestGroup();
    ACFG:RegisterOptionsTable("DailyQuestHelper", quest_options);
	ACFG:RegisterOptionsTable("DailyQuestHelper Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db))
	general_options = DailyQuestHelper:GetGeneralOptions();
	ACFG:RegisterOptionsTable("DailyQuestHelper General", general_options);
	
	local ACD = LibStub("AceConfigDialog-3.0");    
	ACD:AddToBlizOptions("DailyQuestHelper", "DailyQuestHelper");
    ACD:SetDefaultSize("DailyQuestHelper", 700, 400);
	ACD:AddToBlizOptions("DailyQuestHelper General", "General", "DailyQuestHelper");
	ACD:AddToBlizOptions("DailyQuestHelper Profiles", "Profiles", "DailyQuestHelper");
	
    self:RegisterChatCommand("DailyQuestHelper", "ChatCommand");
    self:RegisterChatCommand("dqh", "ChatCommand");
    DailyQuestHelper.DEBUG = false;
end

function DailyQuestHelper:ConvertOldDB()
	DailyQuestHelper:ConvertFrom0To1();
	self.db.char.version = self.databaseVersion;
end

function DailyQuestHelper:ConvertFrom0To1()
	local profilequests = self.db.profile.quests;
	local charquests = self.db.char.quests;
	if (charquests) then
		for i, v in pairs(charquests) do
			if (profilequests[i] == nil) then
				profilequests[i] = v;
			end
		end
		self.db.char.quests = nil;	
	end	
end

function DailyQuestHelper:OnProfileChanged(event, database, newProfileKey)
	profileDB = database.profile;
	general_options = DailyQuestHelper:GetGeneralOptions();
	DailyQuestHelper:ReloadQuestOptions();
end

function DailyQuestHelper:ReloadQuestOptions()
	quests = DailyQuestHelper:CreateDynamicQuestGroup();
	quest_options.args = quests.args;
end

function DailyQuestHelper:GetGeneralOptions()
	general = {
		name = L["General"],
		type = "group",
		--inline = true;
		args = {
			activate = {
				name = L["Enable DailyQuestHelper"],
				type = "toggle",
				width = "full",
				desc = L["Enable DailyQuestHelper"],
				get = function()
						return profileDB.enable;
						end,
				set = function(i, v)
						profileDB.enable = v;
						end,
			},
			shownewquests = {
				name = L["Show a Message for new daily quests"],
				type = "toggle",
				width = "full",
				desc = L["Prints a Message to the chat window when DailyQuestHelper learned a new quest."],
				get = function()
					return profileDB.shownewquests;
					end,
				set = function(i, v)
					profileDB.shownewquests = v;
					end,
			},
		},
	}
	return general;
end

----------------------------------------------------------
-- register events
----------------------------------------------------------
function DailyQuestHelper:OnEnable()
    self:RegisterEvent("GOSSIP_SHOW");
    self:RegisterEvent("QUEST_PROGRESS");
    self:RegisterEvent("QUEST_COMPLETE");
    self:RegisterEvent("QUEST_DETAIL");
end


function DailyQuestHelper:OnDisable()
end

----------------------------------------------------------
-- set addon chat commands
----------------------------------------------------------
function DailyQuestHelper:ChatCommand(input)
    if not input or input:trim() == "" then
        LibStub("AceConfigDialog-3.0"):Open("DailyQuestHelper");
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(DailyQuestHelper, "dqh", "DailyQuestHelper", input);
    end
end

----------------------------------------
-- event GOSSIP_SHOW, auto accept Quests
----------------------------------------
function DailyQuestHelper:GOSSIP_SHOW()
    if (profileDB.enable) then
		DailyQuestHelper:CheckForCompletedQuests();
		DailyQuestHelper:CheckForAvailableQuests();
    end
end

--------------------------------------------------------------------------------------------------
-- Checks if there are already finished quests when talking to a npc which offers multiple quests.
--------------------------------------------------------------------------------------------------
function DailyQuestHelper:CheckForCompletedQuests()
	if (self.DEBUG) then
		DEFAULT_CHAT_FRAME:AddMessage("DEBUG(CheckForCompletedQuests())");
	end
	-- get all active quests this npc has. active quests are quests that are already accepted and in the quest log.
	local activeQuests = {GetGossipActiveQuests()};
	
	-- get the number of active quests from the npc. this is needed to calculate the index of the quest.
	local numOfAcQuests = GetNumGossipActiveQuests();
	
	-- if there are no active quests, than there is nothing to do.
	if (numOfAcQuests > 0) then
		--## DEBUGCODE BEGIN ##--
		if (self.DEBUG) then
			DEFAULT_CHAT_FRAME:AddMessage("DEBUG(AcceptQuest()): Number of Active Quest "..numOfAcQuests);
		end
		--## DEBUGCODE END   ##--
		
		-- go through all active quests and check if this quest is already known and is enabled in the options panel.
		-- When the quest is known and enabled, then check if the quest is already completed. If it is completed, then select
		-- the quest from the npc. After selection the even QUEST_PROGRESS will be fired.
		for numberOfQuestToAnalyse = 1, numOfAcQuests, 1 do
			local indexOfQuest = (numberOfQuestToAnalyse-1)*4;
			if (profileDB.quests[activeQuests[indexOfQuest + 1]] and profileDB.quests[activeQuests[indexOfQuest + 1]]) then
				if (activeQuests[indexOfQuest +4]) then
					SelectGossipActiveQuest(numberOfQuestToAnalyse);					
					if (self.DEBUG) then
						DEFAULT_CHAT_FRAME:AddMessage("DEBUG(CheckForCompletedQuests()): SelectGossipActiveQuest :"..numberOfQuestToAnalyse);
					end
					-- this break is a workaround. Here is a timing problem.
					-- When a quest is selected for completion, then already another quest could be selected, before the first quest was
					-- successfully completed, then the UI stucks.
					break;
				end
			end				
		end
	end
end

-------------------------------------------------------------------------------------------
-- Checks if there are quests available when talking to a npc which offers multiple quests.
-------------------------------------------------------------------------------------------
function DailyQuestHelper:CheckForAvailableQuests()
	if (self.DEBUG) then
		DEFAULT_CHAT_FRAME:AddMessage("DEBUG(CheckForAvailableQuests())");
	end
	-- All available quests from the npc
	local npcQuests = {GetGossipAvailableQuests()};
	
	-- Get the number of available quests. This is needed to calculate the index of the quest.
	local numberOfQuests = GetNumGossipAvailableQuests();
	--## DEBUGCODE BEGIN ##--
	if (self.DEBUG) then
		DEFAULT_CHAT_FRAME:AddMessage("DEBUG(CheckForAvailableQuests()): Number of Quest "..numberOfQuests);
	end
	--## DEBUGCODE END ##--
	
	-- Go through all available quests and check if one or more the offered quests are daily quests. When the npc offers
	-- a daily quest, then add this quest to the database. Also select the quest and skip the dialog further dialog and accept the quest.
	for numberOfQuestToAnalyse = 1, numberOfQuests, 1 do
		-- title, level, isLowLevel, isDaily, isRepeatable are the resutlts for a quest.
		local indexOfQuest = (numberOfQuestToAnalyse-1)*5;
		--## DEBUGCODE BEGIN ##--
		if (self.DEBUG) then
				DEFAULT_CHAT_FRAME:AddMessage("DEBUG(CheckForAvailableQuests()): Index of Quest: "..indexOfQuest);
		end
		--## DEBUGCODE END ##--
		
		if (npcQuests[indexOfQuest + 4]) then
		local questTitle = npcQuests[indexOfQuest + 1];
			--## DEBUGCODE BEGIN ##--
			if (self.DEBUG) then
				DEFAULT_CHAT_FRAME:AddMessage("DEBUG(CheckForAvailableQuests()): Quest is Daily "..questTitle);
			end
			--## DEBUGCODE END ##--
			
			if (profileDB.quests[questTitle] == nil) then
				DailyQuestHelper:AddQuestsToDB(questTitle);
			end
			if (profileDB.quests[questTitle].active) then
				SelectGossipAvailableQuest(numberOfQuestToAnalyse);
				CompleteQuest();
				if (self.DEBUG) then
					DEFAULT_CHAT_FRAME:AddMessage("DEBUG(CheckForAvailableQuests()): Quest completed "..questTitle);
				end
			end
		end
	end
end

---------------------------------------------
-- event Quest_COMPLETE, auto complete Quests
---------------------------------------------
function DailyQuestHelper:QUEST_COMPLETE()
    if (profileDB.enable) then
	--## DEBUGCODE BEGIN ##--
		if (self.DEBUG) then
		  DEFAULT_CHAT_FRAME:AddMessage("DEBUG(QUEST_COMPLETE()): questEvent: QUEST_COMPLETE");
		end
		--## DEBUGCODE END ##--
		
		if (QuestIsDaily()) then
			-- autocomplete quest		
			if (GetNumQuestChoices() > 1) then
			-- if there are more items to choose of, then don't autoturnin
			else
				GetQuestReward(0);
				if (self.DEBUG) then
					DEFAULT_CHAT_FRAME:AddMessage("DEBUG(QUEST_COMPLETE()): GetQuestReward(0)");
				end
			end
		end
	end
end
 
-----------------------------------------
-- event QUEST_DETAIL, auto accept Quests
-----------------------------------------
function DailyQuestHelper:QUEST_DETAIL()
	if (self.DEBUG) then
		DEFAULT_CHAT_FRAME:AddMessage("DEBUG(QUEST_DETAIL())");
	end
    if profileDB.enable then
		-- Check if the shown quest is a daily quest.
		if (QuestIsDaily()) then
			-- When the quest is a daily quest, then check if the quest is already known by DailyQuestHelper.
			-- If it is an unknown quest to DailyQuestHelper, then lern this quest.
			-- Afterwards the quest will be accepted.
			local questTitle = GetTitleText();
			if (profileDB.quests[questTitle] == nil) then
				DailyQuestHelper:AddQuestsToDB(questTitle);
			end			
			if (profileDB.quests[questTitle].active) then
				DailyQuestHelper:AddQuestReward(questTitle);
				AcceptQuest();
				if (self.DEBUG) then
					DEFAULT_CHAT_FRAME:AddMessage("DEBUG(QUEST_DETAIL()): AcceptQuest "..questTitle);
				end
			end
		end
	end
end
------------------------------------------------------
-- Adds a questreward to the db
------------------------------------------------------
function DailyQuestHelper:AddQuestReward(questTitle)
	if (not profileDB.quests[questTitle].reward) then
		profileDB.quests[questTitle].reward = {};
		local numberOfQuestChoices = GetNumQuestChoices();		
		for index = 1,numberOfQuestChoices,1 do
			local name = GetQuestItemInfo("choice", index);			
			profileDB.quests[questTitle].reward[name] = index;
		end
	end
end

----------------------------------------------------------
-- adds a new daily quest to the db
----------------------------------------------------------
function DailyQuestHelper:AddQuestsToDB(questTitle)
	if (profileDB.shownewquests) then
		DEFAULT_CHAT_FRAME:AddMessage("|cffFF7D0ADailyQuestHelper - "..L["Added a new Quest"]..": "..questTitle.."|r");
	end
	profileDB.quests[questTitle] = {};
	profileDB.quests[questTitle].active = true;
	profileDB.quests[questTitle].location = GetRealZoneText();
	DailyQuestHelper:AddQuestReward(questTitle);
	DailyQuestHelper:ReloadQuestOptions()
end

-------------------------------------------
-- event QUEST_PROGRESS, auto accept Quests
-------------------------------------------
function DailyQuestHelper:QUEST_PROGRESS()
		if (self.DEBUG) then
		  DEFAULT_CHAT_FRAME:AddMessage("DEBUG(AcceptQuest()): questEvent: QUEST_PROGRESS");
		end
    if profileDB.enable then
			--## DEBUGCODE BEGIN ##--
		if (self.DEBUG) then
		  DEFAULT_CHAT_FRAME:AddMessage("DEBUG(AcceptQuest()): questEvent: QUEST_PROGRESS");
		end
		--## DEBUGCODE END ##--
		-- Check if the quest is a daily quest
		if (QuestIsDaily()) then
			-- check if the quest offered quest is already completed. If the quest is completed, then skip the dialog and jump to the
			-- complete quest dialog.
			if (IsQuestCompletable()) then
				--## DEBUGCODE BEGIN ##--
				if (self.DEBUG) then
					DEFAULT_CHAT_FRAME:AddMessage("DEBUG(QuestProgress()): Quest is completable.");				
				end
				--## DEBUGCODE END ##--
		
				CompleteQuest();
			else
				--## DEBUGCODE BEGIN ##--
				if (self.DEBUG) then
					DEFAULT_CHAT_FRAME:AddMessage("DEBUG(QuestProgress()): Quest not completable.");
				end
				--## DEBUGCODE END ##--
			end
		end
    end
end

----------------------------------------------------------
-- Creates the option panel for quests
----------------------------------------------------------
function DailyQuestHelper:CreateDynamicQuestGroup()
	local quests = {
		name = L["Quests"],
		type = "group",
		args = {},
		}
	-- iterate through all learned quests
	for questKey, questValues in pairs(profileDB.quests) do
		-- get the location from the quest and check if this location is already a group in the quest table for the option panel.
		-- the location is used to group the quests within the options panel
		local zonename = questValues.location;
		if (quests.args[zonename] == nil) then
			quests.args[zonename] = {
				name = zonename,
				type = "group",
				inline = true,
				args = {},
			}
		end
		
		-- create a toggle for the quest within to location group.
		quests.args[zonename].args[questKey] = {};
		local toggle = quests.args[zonename].args[questKey];
		toggle.name = questKey;

		toggle.type = "toggle";
		local description = "";

		toggle.desc = description;
		toggle.get = function()
						return profileDB.quests[questKey].active;							
					end
		toggle.set = function(i, v)
						profileDB.quests[questKey].active = v;
					end
	end
	return quests;
end